'use strict';
/**
 * 项目地址：https://github.com/hunshcn/gh-proxy
 **/
const ASSET_URL = 'https://page.futv.us.kg/';
// 前缀，如果自定义路由为example.com/gh/*，将PREFIX改为 '/gh/'，注意，少一个杠都会错！
const PREFIX = '/';
// 分支文件使用jsDelivr镜像的开关，0为关闭，默认关闭
const Config = {
    jsdelivr: 0
};

const whiteList = []; // 白名单

const urlPatterns = [
    /^(?:https?:\/\/)?github\.com\/.+?\/.+?\/(?:releases|archive)\/.*$/i,
    /^(?:https?:\/\/)?github\.com\/.+?\/.+?\/(?:blob|raw)\/.*$/i,
    /^(?:https?:\/\/)?github\.com\/.+?\/.+?\/(?:info|git-).*$/i,
    /^(?:https?:\/\/)?raw\.(?:githubusercontent|github)\.com\/.+?\/.+?\/.+?\/.+$/i,
    /^(?:https?:\/\/)?gist\.(?:githubusercontent|github)\.com\/.+?\/.+?\/.+$/i,
    /^(?:https?:\/\/)?github\.com\/.+?\/.+?\/tags.*$/i
];

const PREFLIGHT_INIT = {
    status: 204,
    headers: new Headers({
        'access-control-allow-origin': '*',
        'access-control-allow-methods': 'GET,POST,PUT,PATCH,TRACE,DELETE,HEAD,OPTIONS',
        'access-control-max-age': '1728000',
    }),
};

function makeRes(body, status = 200, headers = {}) {
    headers['access-control-allow-origin'] = '*';
    return new Response(body, { status, headers });
}

function newUrl(urlStr) {
    try {
        return new URL(urlStr);
    } catch {
        return null;
    }
}

addEventListener('fetch', e => {
    const ret = fetchHandler(e)
        .catch(err => makeRes('cfworker error:\n' + err.stack, 502));
    e.respondWith(ret);
});

async function fetchHandler(e) {
    const req = e.request;
    const cacheKey = new Request(req.url, { method: 'GET' });
    const cache = caches.default;

    let cachedResponse = await cache.match(cacheKey);
    if (cachedResponse) {
        return cachedResponse; // 如果缓存命中，直接返回
    }

    const urlStr = req.url;
    const urlObj = new URL(urlStr);
    let path = urlObj.searchParams.get('q');

    if (path) {
        return Response.redirect('https://' + urlObj.host + PREFIX + path, 301);
    }

    path = urlObj.href.substr(urlObj.origin.length + PREFIX.length).replace(/^https?:\/+/, 'https://');
    if (urlPatterns.some(pattern => path.search(pattern) === 0)) {
        try {
            const response = await httpHandler(req, path);
            e.waitUntil(cache.put(cacheKey, response.clone())); // 缓存响应
            return response;
        } catch (error) {
            console.error(`Error handling request: ${error.message}`);
            return makeRes('Internal Server Error', 500);
        }
    } else {
        return fetch(ASSET_URL + path);
    }
}

async function httpHandler(req, pathname) {
    const reqHdrRaw = req.headers;

    // preflight
    if (req.method === 'OPTIONS' && reqHdrRaw.has('access-control-request-headers')) {
        return new Response(null, PREFLIGHT_INIT);
    }

    const reqHdrNew = new Headers(reqHdrRaw);
    let allowed = whiteList.length === 0 || whiteList.some(item => pathname.includes(item));
    if (!allowed) {
        return new Response("blocked", { status: 403 });
    }

    if (!pathname.startsWith('https://')) {
        pathname = 'https://' + pathname;
    }
    const urlObj = newUrl(pathname);

    const reqInit = {
        method: req.method,
        headers: reqHdrNew,
        redirect: 'manual',
        body: req.body
    };
    return proxy(urlObj, reqInit);
}

async function proxy(urlObj, reqInit) {
    const res = await fetch(urlObj.href, reqInit);
    const resHdrNew = new Headers(res.headers);

    const status = res.status;

    if (resHdrNew.has('location')) {
        let _location = resHdrNew.get('location');
        if (checkUrl(_location)) {
            resHdrNew.set('location', PREFIX + _location);
        } else {
            reqInit.redirect = 'follow';
            return proxy(newUrl(_location), reqInit);
        }
    }

    resHdrNew.set('access-control-expose-headers', '*');
    resHdrNew.set('access-control-allow-origin', '*');

    // 删除不必要的头部
    resHdrNew.delete('content-security-policy');
    resHdrNew.delete('content-security-policy-report-only');
    resHdrNew.delete('clear-site-data');

    return new Response(res.body, {
        status,
        headers: resHdrNew,
    });
}

function checkUrl(u) {
    return urlPatterns.some(pattern => u.search(pattern) === 0);
}